from __future__ import barry_as_FLUFL

import asyncio
import sys
from typing import *

from . import conjugate
from . import options
from . import translate


def main() -> None:
    asyncio.run(main_())


async def main_() -> None:
    subcommand = sys.argv[1:2]
    if subcommand == ["translate"]:
        await translate.main(sys.argv[2:])
    elif subcommand == ["conjugate"]:
        await conjugate.main(sys.argv[2:])
    else:
        print(options.__doc__)

if __name__ == "__main__":
    main()



import asyncio
import itertools as it
from typing import *

import aiohttp
from bs4 import BeautifulSoup
from tabulate import tabulate


def get_conjugate_url(word: str) -> str:
    return f"https://www.spanishdict.com/conjugate/{word}"


async def r_get(url: str, session: aiohttp.ClientSession) -> aiohttp.client_reqrep.ClientResponse:
    return await session.request(method="GET", url=url)


async def get_participles(word: str, session: aiohttp.ClientSession) -> Dict[str, str]:
    try:
        r = await r_get(get_conjugate_url(word), session)
        soup = BeautifulSoup(await r.text(), "html.parser")
        # the table right above the indicative conjugation
        participles = soup.find("table", {"class": "participlesTable--9jeAZFrU"})
        culled = participles.find_all("a")
        present = str(culled[1].find("span").text)
        past = str(culled[-1].find("span").text)
        return {
            "Present": "None" if "<span" in present else present,
            "Past": "None" if "<span" in past else past
        }
    except Exception:
        return {"Present": "", "Past": ""}


def get_aria_of_tag(tag: BeautifulSoup) -> Optional[str]:
    try:
        return tag.get("aria-label")
    # Handle if tag is None
    except AttributeError:
        return None


def get_column_names(soup: BeautifulSoup) -> List[str]:
    return  [
        c.find('a').contents[0]
        # first tr second td > div > div
        for c in soup.find_all("div", {"class": "inlineTooltipWrapper--14F1yy0t"})
    ]


def get_conjugations_from_table(
        soup: BeautifulSoup) -> Dict[str, List[str]]:

    raw = [
        get_aria_of_tag(v.find('a'))
        # td > div
        for v in soup.find_all("div", {"class": "vtableWordContents--1fU54Tfk"})
    ]

    tenses = get_column_names(soup)
    result = dict()
    result[''] = ["yo", "tú", "Ud.",
                  "nosotros", "vosotros",
                  "Uds."]
    result.update({tense: [] for tense in tenses})

    for tense, value in zip(it.cycle(tenses), raw):
        result[tense].append(value)

    return result


async def get_conjugations(
        word: str, mood: int,
        session: aiohttp.ClientSession
        ) -> Dict[str, List[str]]:

    try:
        r = await r_get(get_conjugate_url(word), session)
        soup = BeautifulSoup(await r.text(), "html.parser")
        # #conjugation-content-wrapper > div > div > div > table
        conjugations = soup.find_all(
                "table", {"class": "vtable--2WLTGmgs"})
        return get_conjugations_from_table(conjugations[mood])
    except Exception:
        return {"": [""]}


async def main(args):
    async with aiohttp.ClientSession() as session:
        if args[0] in {'-p', "--participle"}:
            words = args[1:]
            parts = await asyncio.gather(*(get_participles(word, session) for word in words))
            print(tabulate({
                "Original": words,
                "Present": [p["Present"] for p in parts],
                "Past": [p["Past"] for p in parts]},
                headers="keys", tablefmt="fancy_grid"))

        else:
            try:
                mood = {
                    "indicative": 0,
                    "subjunctive": 1,
                    "imperative": 2,
                    "continuous": 3,
                    "perfect": 4,
                    "perfect-subjunctive": 5
                }[args[0].lower()]
                words = args[1:]
            except (IndexError, KeyError):
                mood = 0
                words = args[:]

            conjugations = await asyncio.gather(
                    *(get_conjugations(w, mood, session) for w in words))

            to_print = []
            for w, conj in zip(words, conjugations):
                table = tabulate(
                    conj, headers="keys", tablefmt="fancy_grid"
                )

                # Printing the word in a caption on the table
                top = table[:table.find("\n")]
                newtop = top[0] + top[1]*(len(top) - 2) + top[-1]
                spaces = ' '*((len(top)-len(w)) // 2 - 1)
                begspace = ' ' * ((len(top)-len(w)) % 2)

                to_print.append('\n'.join((
                    newtop,
                    f"|{spaces}{w}{spaces}{begspace}|",
                    table.replace(top[0], '╞').replace("═╕", '═╡')
                )))

            print('\n\n'.join(to_print))




"""Spanishdict cli

  Usage:
      python spanishdict <subcommand>
  
  Subcommands:
      translate [words]
      conjugate FLAGS [mood] [words]
          Flags:
              -p, --participle get the particple of verbs
          Mood:
              indicative(default)
              subjunctive
              imperative
              continuous
              perfect
              perfect-subjunctive
"""



from __future__ import barry_as_FLUFL

import asyncio
import sys
from typing import *

import conjugate
import options
import translate


async def main() -> None:
    subcommand = sys.argv[1:2]
    if subcommand == ["translate"]:
        await translate.main(sys.argv[2:])
    elif subcommand == ["conjugate"]:
        await conjugate.main(sys.argv[2:])
    else:
        print(options.__doc__)

if __name__ == "__main__":
    asyncio.run(main())




from __future__ import barry_as_FLUFL

import asyncio
import sys
import time
from typing import *

import aiohttp
from bs4 import BeautifulSoup
from tabulate import tabulate


url = "https://www.spanishdict.com"


async def r_get(url: str, session: aiohttp.ClientSession) -> aiohttp.client_reqrep.ClientResponse:
    """Asyncronous get"""
    return await session.request(method="GET", url=url)


async def translate(word: str, session: aiohttp.ClientSession) -> List[str]:
    """
    Translates a word to english or spanish.

    :param word: the english or spanish word to translate
    :return: the possible translations
    """
    try:
        r = await r_get(f"{url}/translate/{word}", session)
        soup = BeautifulSoup(await r.text(), "html.parser")
        # translation_div > _ > _ > quickdef
        translation_div = soup.find("div", {"class": "quickdefsWrapperDesktop--fy8K_XyV"})
        # quickdef > translations
        res = [element.contents[0]
                for element in
                translation_div.find_all('a', {"class": "a--3Le9u96E"})]
        return res
    except Exception:
        return ["TODO"]


async def mass_translate(words: List[str]) -> List[List[str]]:
    """
    Mass translate words to english or spanish.

    :param words: the english or spanish words to translate
    :return: the list of translations
    """
    async with aiohttp.ClientSession() as session:
        return await asyncio.gather(*(translate(word, session) for word in words))


async def main(words: List[str]) -> None:
    raw = words[0] == "--raw"
    try:
        translations = await mass_translate(words[1:] if raw else words)
    except Exception as e:
        translations = [[]] * len(words)

    if raw:
        print(*(','.join(t) for t in translations), sep='\n')
    else:
        print(tabulate({
            "Original": words,
            "Translated": [', '.join(t) for t in translations]},
            headers="keys", tablefmt="fancy_grid"))


if __name__ == "__main__":
    asyncio.run(main(sys.argv[1:]))

